@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -40,8 +40,8 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    in = fopen(argv[1],"r");
-    out = fopen(argv[2],"w");
+    in = fopen(argv[1],"rb");
+    out = fopen(argv[2],"wb");
 
     /* open input and output */
     if (!in || !out) {
