@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -41,7 +41,7 @@ int main(int argc, char** argv)
     ofilename=argv[2];
 
     /* open input file */
-    in = fopen(filename,"r");
+    in = fopen(filename,"rb");
     if(!in) {
         fprintf(stderr, "ERROR: unable to open input file %s\n",filename);
         return 1;
