@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -695,6 +695,7 @@ static int read_BUFR(reader *r)
     }
 
     if(length==0) {
+        grib_buffer_delete(c,buf);
         return GRIB_INVALID_MESSAGE;
     }
 
@@ -800,10 +801,10 @@ static int read_BUFR(reader *r)
         break;
       default :
         r->seek_from_start(r->read_data,r->offset+4);
+        grib_buffer_delete(c,buf);
         return GRIB_UNSUPPORTED_EDITION;
     }
 
-
     /* Assert(i <= sizeof(tmp)); */
     err=read_the_rest(r, length, tmp, i, 1);
     if (err)
@@ -901,9 +902,9 @@ static int read_any_gts(reader *r)
     int err = 0;
     unsigned char* buffer=NULL;
     unsigned long magic = 0;
-    unsigned long start = 0x010d0d0a;
-    unsigned long theEnd = 0x0d0d0a03;
-    unsigned char tmp[128]={0,}; /* See ECC-735 */
+    unsigned long start = 0x010d0d0a;  /* SOH CR CR LF */
+    unsigned long theEnd = 0x0d0d0a03; /* CR CR LF ETX */
+    unsigned char tmp[1024]={0,}; /* See ECC-735 */
     size_t message_size=0;
     size_t already_read=0;
     int i=0;
