@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2017 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -9,10 +9,11 @@
  */
 
 #include "grib_api_internal.h"
+static int fail_on_error = 1;
 
 static void usage(const char* prog)
 {
-    printf("usage: %s [-v] infile1 infile2 ... \n",prog);
+    printf("Usage: %s [-v] [-f] infile1 infile2 ... \n",prog);
     exit(1);
 }
 
@@ -30,11 +31,32 @@ static int count_messages(FILE* in, int message_type, unsigned long *count)
     /* printf("message_type=%d\n", message_type); */
     if (message_type == CODES_GRIB)        wmo_read=wmo_read_grib_from_file_malloc;
     else if (message_type == CODES_BUFR)   wmo_read=wmo_read_bufr_from_file_malloc;
+    else if (message_type == CODES_GTS)    wmo_read=wmo_read_gts_from_file_malloc;
     else                                   wmo_read=wmo_read_any_from_file_malloc;
-
-    while ( (mesg=wmo_read(in,0,  &size,&offset,&err))!=NULL && err==GRIB_SUCCESS) {
-        grib_context_free(c,mesg);
-        (*count)++;
+    
+    if (fail_on_error)
+    {
+        while ( (mesg=wmo_read(in,0,  &size,&offset,&err))!=NULL && err==GRIB_SUCCESS) {
+            grib_context_free(c,mesg);
+            (*count)++;
+        }
+    }
+    else
+    {
+        int done = 0;
+        while(!done) {
+            mesg = wmo_read(in, 0, &size, &offset, &err);
+            /*printf("Count so far=%ld, mesg=%x, err=%d (%s)\n", *count, mesg, err, grib_get_error_message(err));*/
+            if (!mesg) {
+                if (err == GRIB_END_OF_FILE || err == GRIB_PREMATURE_END_OF_FILE) {
+                    done = 1; /* reached the end */
+                }
+            }
+            if (mesg && !err) {
+                grib_context_free(c,mesg);
+                (*count)++;
+            }
+        }
     }
 
     if (err==GRIB_END_OF_FILE) err=GRIB_SUCCESS;
@@ -47,7 +69,7 @@ int main(int argc,char* argv[])
     FILE* infh = NULL;
     char* filename;
     int i, verbose=0;
-    int err=0;
+    int err=0, files_processed=0;
     unsigned long count_total=0, count_curr=0;
     int message_type = 0; /* GRIB, BUFR etc */
 
@@ -55,24 +77,31 @@ int main(int argc,char* argv[])
     
     if (strstr(argv[0], "grib_count")) message_type = CODES_GRIB;
     if (strstr(argv[0], "bufr_count")) message_type = CODES_BUFR;
+    if (strstr(argv[0], "gts_count"))  message_type = CODES_GTS;
 
     count_total=0;
     for (i=1;i<argc;i++) {
         if (strcmp(argv[i], "-v")==0) {
             verbose = 1;
-            if (argc <3) usage(argv[0]);
+            continue;
+        }
+        if (strcmp(argv[i], "-f")==0) {
+            fail_on_error = 0;
             continue;
         }
         filename=argv[i];
-        infh=fopen(filename,"r");
+        if (strcmp(filename,"-")==0)
+            infh=stdin;
+        else
+            infh=fopen(filename,"rb");
         if (!infh) {
             perror(filename);
             exit(1);
         }
-
+        files_processed=1; /* At least one file processed */
         count_curr=0;
         err=count_messages(infh, message_type, &count_curr);
-        if (err) {
+        if (err && fail_on_error) {
             fprintf(stderr,"Invalid message(s) found in %s", filename);
             if (count_curr>0) fprintf(stderr," (got as far as %lu)", count_curr);
             fprintf(stderr,"\n");
@@ -88,6 +117,8 @@ int main(int argc,char* argv[])
 
         fclose(infh);
     }
+
+    if (!files_processed) usage(argv[0]);
     if (verbose) {
         printf("%7lu %s\n", count_total, "total");
     } else {
