@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -167,7 +167,9 @@ int grib_tool(int argc, char **argv)
         dump_file=stdout;
     }
 
-    if (is_index_file(global_options.infile->name) &&
+    /* ECC-926: Currently only GRIB indexing works. Disable the through_index if BUFR, GTS etc */
+    if (global_options.mode == MODE_GRIB &&
+        is_index_file(global_options.infile->name) &&
             ( global_options.infile_extra && is_index_file(global_options.infile_extra->name))) {
         global_options.through_index=1;
         return grib_tool_index(&global_options);
@@ -219,9 +221,13 @@ static int grib_tool_with_orderby(grib_runtime_options* options)
     }
 
     options->handle_count=0;
+    grib_context_set_handle_file_count(c, 0); /* ECC-873 */
+    grib_context_set_handle_total_count(c, 0); /* ECC-873 */
     while(!options->skip_all && ((h = grib_fieldset_next_handle(set,&err))
             != NULL || err != GRIB_SUCCESS )) {
         options->handle_count++;
+        grib_context_set_handle_file_count(c, options->handle_count);/* ECC-873 */
+        grib_context_set_handle_total_count(c, options->handle_count);/* ECC-873 */
         options->error=err;
 
         if (!h) {
@@ -289,7 +295,7 @@ static int grib_tool_without_orderby(grib_runtime_options* options)
         if (strcmp(infile->name,"-")==0)
             infile->file = stdin;
         else
-            infile->file = fopen(infile->name,"r");
+            infile->file = fopen(infile->name,"rb");
         if(!infile->file) {
             perror(infile->name);
             exit(1);
@@ -532,38 +538,32 @@ static int scan(grib_context* c,grib_runtime_options* options,const char* dir)
             process(c,options,buf);
         }
     }
+    closedir(d);
     return 0;
 }
 #else
-static int isWinDir(const struct _finddata_t *fileinfo)
-{
-    if((fileinfo->attrib & 16) == 16)
-        return 1;
-    return 0;
-}
-static void doProcessing(grib_context* c,grib_runtime_options* options,const char* dir, const struct _finddata_t *fileinfo)
-{
-    if(isWinDir(fileinfo))
-    {
-        if(strcmp(fileinfo->name, ".") != 0 && strcmp(fileinfo->name,"..") != 0) {
-            char buf[1024];
-            sprintf(buf,"%s/%s",dir,fileinfo->name);
-            process(c,options,buf);
-        }
-    }
-}
 static int scan(grib_context* c,grib_runtime_options* options,const char* dir) {
     struct _finddata_t fileinfo;
     intptr_t handle;
-    if((handle = _findfirst(dir, &fileinfo)) != -1)
-        doProcessing(c, options, dir, &fileinfo);
+    char buffer[1024];
+    sprintf(buffer,  "%s/*", dir);
+    if((handle = _findfirst(buffer, &fileinfo)) != -1)
+    {
+        do {
+            if(strcmp(fileinfo.name, ".") != 0 && strcmp(fileinfo.name,"..") != 0) {
+                char buf[1024];
+                sprintf(buf, "%s/%s", dir, fileinfo.name);
+                process(c, options, buf);
+            }
+        } while(!_findnext(handle, &fileinfo));
+
+        _findclose(handle);
+    }
     else
     {
         grib_context_log(c,(GRIB_LOG_ERROR) | (GRIB_LOG_PERROR) , "opendir %s",dir);
         return GRIB_IO_PROBLEM;
     }
-    while(!_findnext(handle, &fileinfo))
-        doProcessing(c, options, dir, &fileinfo);
     return 0;
 }
 #endif
